@@ -48,7 +48,7 @@ where
         } else {
             return Err(TypedHeaderRejection {
                 name: T::name(),
-                reason: Reason::Missing,
+                reason: TypedHeaderRejectionReason::Missing,
             });
         };
 
@@ -56,11 +56,11 @@ where
             Ok(Some(value)) => Ok(Self(value)),
             Ok(None) => Err(TypedHeaderRejection {
                 name: T::name(),
-                reason: Reason::Missing,
+                reason: TypedHeaderRejectionReason::Missing,
             }),
             Err(err) => Err(TypedHeaderRejection {
                 name: T::name(),
-                reason: Reason::Error(err),
+                reason: TypedHeaderRejectionReason::Error(err),
             }),
         }
     }
@@ -80,12 +80,27 @@ impl<T> Deref for TypedHeader<T> {
 #[derive(Debug)]
 pub struct TypedHeaderRejection {
     name: &'static http::header::HeaderName,
-    reason: Reason,
+    reason: TypedHeaderRejectionReason,
 }
 
+impl TypedHeaderRejection {
+    /// Name of the header that caused the rejection
+    pub fn name(&self) -> &http::header::HeaderName {
+        self.name
+    }
+
+    /// Reason why the header extraction has failed
+    pub fn reason(&self) -> &TypedHeaderRejectionReason {
+        &self.reason
+    }
+}
+
+/// Additional information regarding a [`TypedHeaderRejection`](super::TypedHeaderRejection)
 #[derive(Debug)]
-enum Reason {
+pub enum TypedHeaderRejectionReason {
+    /// The header was missing from the HTTP request
     Missing,
+    /// An error occured when parsing the header from the HTTP request
     Error(headers::Error),
 }
 
@@ -103,10 +118,10 @@ impl IntoResponse for TypedHeaderRejection {
 impl std::fmt::Display for TypedHeaderRejection {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match &self.reason {
-            Reason::Missing => {
+            TypedHeaderRejectionReason::Missing => {
                 write!(f, "Header of type `{}` was missing", self.name)
             }
-            Reason::Error(err) => {
+            TypedHeaderRejectionReason::Error(err) => {
                 write!(f, "{} ({})", err, self.name)
             }
         }
@@ -116,8 +131,8 @@ impl std::fmt::Display for TypedHeaderRejection {
 impl std::error::Error for TypedHeaderRejection {
     fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
         match &self.reason {
-            Reason::Error(err) => Some(err),
-            Reason::Missing => None,
+            TypedHeaderRejectionReason::Error(err) => Some(err),
+            TypedHeaderRejectionReason::Missing => None,
         }
     }
 }
@@ -137,21 +152,14 @@ mod tests {
 
         let app = Router::new().route("/", get(handle));
 
-        let addr = run_in_background(app).await;
+        let client = TestClient::new(app);
 
-        let client = reqwest::Client::new();
-
-        let res = client
-            .get(format!("http://{}", addr))
-            .header("user-agent", "foobar")
-            .send()
-            .await
-            .unwrap();
-        let body = res.text().await.unwrap();
+        let res = client.get("/").header("user-agent", "foobar").send().await;
+        let body = res.text().await;
         assert_eq!(body, "foobar");
 
-        let res = client.get(format!("http://{}", addr)).send().await.unwrap();
-        let body = res.text().await.unwrap();
+        let res = client.get("/").send().await;
+        let body = res.text().await;
         assert_eq!(body, "Header of type `user-agent` was missing");
     }
 }
